@@ -4,8 +4,8 @@
 
 ;; Author: Michał Krzywkowski <k.michal@zoho.com>
 ;; Keywords: c, tools
-;; Package-Version: 20251020.1212
-;; Package-Revision: efad3b9f9209
+;; Package-Version: 20251229.1015
+;; Package-Revision: 95daa680767b
 ;; Homepage: https://github.com/mkcms/compiler-explorer.el
 ;; Package-Requires: ((emacs "28.1")
 ;;                    (plz "0.9")
@@ -29,12 +29,12 @@
 ;;; Commentary:
 
 ;;
-;;; compiler-explorer.el
+;; * compiler-explorer.el
 ;;
-;; Package that provides a simple client for https://godbolt.org service.
+;; Package that provides a client for https://godbolt.org service.
 ;;
 ;;
-;;; Usage
+;;; ** Usage
 ;;
 ;; M-x `compiler-explorer' is the main entry point.  It will ask you for a
 ;; language and display source&compilation buffers.  Type something in the
@@ -42,6 +42,9 @@
 ;; compiled asm code.  Another buffer displays output of the compiled and
 ;; executed program.
 ;;
+;;
+;; *** Compilation
+;;
 ;; M-x `compiler-explorer-set-compiler' changes the compiler for current
 ;; session.
 ;;
@@ -51,18 +54,41 @@
 ;; it to current compilation.  M-x `compiler-explorer-remove-library'
 ;; removes them.
 ;;
+;;
+;; *** Execution
+;;
 ;; M-x `compiler-explorer-set-execution-args' sets the arguments for the
 ;; executed program.
 ;;
 ;; M-x `compiler-explorer-set-input' reads a string from minibuffer that
 ;; will be used as input for the executed program.
 ;;
+;;
+;; *** Session management
+;;
+;; M-x `compiler-explorer-new-session' kills the current session and
+;; creates a new one, asking for source language.
+;;
+;; M-x `compiler-explorer-previous-session' lets you restore previous
+;; sessions.
+;;
+;; M-x `compiler-explorer-discard-session' kills the current or selected
+;; sessions and forgets about them forever.
+;;
+;; M-x `compiler-explorer-exit' kills the current session.
+;;
+;;
+;; *** ASM
+;;
 ;; M-x `compiler-explorer-browse-opcode-documentation' opens a website
 ;; that contains the documentation for the opcode at point.
 ;;
 ;; M-x `compiler-explorer-jump' jumps to ASM block for the source line at
 ;; point and vice versa.
 ;;
+;;
+;; *** Tools
+;;
 ;; M-x `compiler-explorer-add-tool' asks for the name of a tool, adds it
 ;; to current compilation and displays a new buffer showing the tool's
 ;; output.
@@ -76,18 +102,13 @@
 ;; M-x `compiler-explorer-set-tool-input' reads a string from minibuffer
 ;; that will be used as input for an added tool.
 ;;
+;;
+;;
+;; *** Other commands
+;;
 ;; M-x `compiler-explorer-load-example' prompts for a name of a builtin
 ;; example and loads it.
 ;;
-;; M-x `compiler-explorer-new-session' kills the current session and
-;; creates a new one, asking for source language.
-;;
-;; M-x `compiler-explorer-previous-session' lets you restore previous
-;; sessions.
-;;
-;; M-x `compiler-explorer-discard-session' kills the current or selected
-;; sessions and forgets about them forever.
-;;
 ;; M-x `compiler-explorer-make-link' generates a link for current
 ;; compilation so it can be opened in a browser and shared.
 ;;
@@ -96,7 +117,39 @@
 ;;
 ;; M-x `compiler-explorer-layout' cycles between different layouts.
 ;;
-;; M-x `compiler-explorer-exit' kills the current session.
+;;
+;;; ** Customization
+;;
+;; Additional customization is possible via M-x `customize-group'
+;; `compiler-explorer'.
+;;
+;;
+;; *** Usage with language servers
+;;
+;; The following snippet sets up the built-in eglot package to start a
+;; language server for each session, and to automatically create and
+;; update a `compile_flags.txt' file (recognized by clangd) to have the
+;; same compiler arguments that are set for the current session.  This
+;; requires that the `compiler-explorer-make-temp-file' custom variable is
+;; non-nil.
+;;
+;;
+;;   (add-hook 'compiler-explorer-new-session-hook #'eglot-ensure)
+;;
+;;   (defun my/compiler-explorer-params-change-hook (what value)
+;;     "Hook run when compilation parameters WHAT change to VALUE."
+;;     (pcase what
+;;       ('compiler-args
+;;        (with-current-buffer compiler-explorer--buffer
+;;          (when (derived-mode-p 'c-mode 'c++-mode)
+;;            (with-temp-file "compile_flags.txt"
+;;              (insert (mapconcat #'identity
+;;                                 (split-string-and-unquote value) "\n")))
+;;            (when (eglot-current-server)
+;;              (eglot-reconnect (eglot-current-server))))))))
+;;
+;;   (add-hook 'compiler-explorer-params-change-hook
+;;             #'my/compiler-explorer-params-change-hook)
 ;;
 ;;
 
@@ -1423,7 +1476,17 @@ It must have been created with `compiler-explorer--current-session'."
                                                       (consp obj)
                                                       (vectorp obj))))))
       (unless (funcall pred val)
-        (error "Invalid %s: %s" sym val)))
+        (error "Invalid %S: %s" sym val)))
+    (dolist (elt libs)
+      (unless (and (consp elt)
+                   (stringp (car elt))
+                   (stringp (cdr elt)))
+        (error "Invalid library: %s" elt)))
+    (dolist (elt tools)
+      (unless (and (listp elt)
+                   (= 3 (length elt))
+                   (cl-every #'stringp elt))
+        (error "Invalid tool: %s" elt)))
 
     (let ((ce--inhibit-request t))
       (ce-new-session lang-name compiler)
@@ -1448,6 +1511,7 @@ It must have been created with `compiler-explorer--current-session'."
 
 (defun ce--save-sessions ()
   "Save all sessions to a file."
+  (remove-hook 'kill-emacs-hook #'ce--save-sessions)
   (let ((current-session (and (ce--active-p) (ce--current-session))))
     (when current-session
       (push current-session ce--session-ring))
@@ -1534,6 +1598,7 @@ It must have been created with `compiler-explorer--current-session'."
   :lighter " CE"
   :keymap ce-mode-map
   :require 'compiler-explorer
+  (add-hook 'kill-emacs-hook #'ce--save-sessions)
   (unless ce-mode
     (ce--cleanup)))
 
@@ -1961,15 +2026,17 @@ all the previous sessions one by one."
   (unless nth
     (setq nth 0))
   (let ((prev (nth nth ce--session-ring))
-        (current (and (ce--session-savable-p) (ce--current-session))))
+        (current (and (ce--session-savable-p) (ce--current-session)))
+        (success nil))
     (setq ce--session-ring (seq-remove-at-position ce--session-ring nth))
 
-    (condition-case nil
+    (condition-case err
         (prog1 t
           (unwind-protect
               (let ((ce--session-ring nil))
                 ;; Override the ring to not mess with it.
-                (ce--restore-session prev))
+                (ce--restore-session prev)
+                (setq success t))
             ;; Insert last session into the ring as the *oldest* item.  We have
             ;; to do this, otherwise we would only be able to cycle between two
             ;; sessions.
@@ -1978,11 +2045,14 @@ all the previous sessions one by one."
 
             ;; Redefine the menu with the ring updated (for "Restore session"
             ;; submenu)
-            (ce--define-menu)))
+            (when success
+              (ce--define-menu))))
       (error
        (ce--cleanup 'skip-save-session)
        (display-warning
-        'compiler-explorer "Previous session appears to be corrupt" :warning)
+        'compiler-explorer
+        (format "Previous session is corrupt: %s" (error-message-string err))
+        :warning)
        nil))))
 
 (defun ce-discard-session (&optional indices interactive)
@@ -2407,8 +2477,6 @@ The hook `compiler-explorer-hook' is always run at the end."
      (t (call-interactively #'ce-new-session))))
   (run-hooks 'ce-hook))
 
-(add-hook 'kill-emacs-hook #'ce--save-sessions)
-
 (provide 'compiler-explorer)
 ;;; compiler-explorer.el ends here
 
